#!/usr/bin/python

import sys

####################    
# util
####################
from subprocess import Popen,PIPE
def call(args,cwd=None,shell=False):
    p = Popen(args, stdout=PIPE, stderr=PIPE, shell=shell, cwd=cwd)
    out,err = p.communicate()
    rc = p.returncode
    return rc,out,err

####################    
# input
####################
try:
    cmd,repo_dir = sys.argv
except:
    print "Usage: %s <repo_dir>" % sys.argv[0]
    sys.exit(1)
    
####################    
# run git log
####################
if 'win' in sys.platform:  
    git_exe = r'C:\Program Files\Git\bin\git.exe' # problem with return codes on msysgit 1.7.0.2
else:
    git_exe = 'git' 
    
rc,out,err = call([git_exe,'log','--all'],cwd=repo_dir)
if rc != 0:
    print >>sys.stderr, 'Failed to execute git log. return code = %s, error:\n%s\n\noutput:\n%s' % (rc,err,out)
    sys.exit(rc)
log_lines = out.split('\n')
    
####################    
# analyze logs
####################
import re
rev_mapping = [] # list of (sha,rev)
sha = None
for i,line in enumerate(log_lines):
    m = re.match(r'commit ([0-9a-fA-F]{40})$',line)
    if m:
        if sha is not None:
            print >>sys.stderr, "Warning: found commit without git-svn-id. sha=%s" % (sha,)
        sha = m.group(1)
        
    m = re.match(r'\s*git-svn-id:.*@([0-9]+)\b',line)
    if m:
        if sha is None:
            print >>sys.stderr, "Warning: found git-svn-id without a previous commit line (with sha) at line %s:\n\t%s" % (i,line)
        else:
            svn_rev = m.group(1)
            rev_mapping.append( (sha,svn_rev) )
            sha = None
if sha is not None:
    print >>sys.stderr, "Warning: last commit has no git-svn-id. sha=%s" % (sha,)

####################    
# output    
####################
for sha,svn_rev in rev_mapping:
    print '%s:%s' % (svn_rev,sha)
